@@ -44,9 +44,14 @@
 int *log_table = NULL;
 int *ilog_table = NULL;
 int *ilog_table_begin = NULL;
+static int init_counter = 0;
 
-void rs_galois_init_tables()
+void rs_galois_init_tables(void)
 {
+  if (init_counter++ > 0) {
+    /* already initialized */
+    return;
+  }
   log_table = (int*)malloc(sizeof(int)*FIELD_SIZE);
   ilog_table_begin = (int*)malloc(sizeof(int)*FIELD_SIZE*3);
   int i = 0;
@@ -65,10 +70,21 @@ void rs_galois_init_tables()
   ilog_table = &ilog_table_begin[GROUP_SIZE];
 }
 
-void rs_galois_deinit_tables()
+void rs_galois_deinit_tables(void)
 {
-  free(log_table);
-  free(ilog_table_begin);
+  init_counter--;
+  if (init_counter < 0) {
+    /* deinit when not initialized?? */
+    init_counter = 0;
+  } else if (init_counter > 0) {
+    /* still at least one desc using it */
+    return;
+  } else {
+    free(log_table);
+    log_table = NULL;
+    free(ilog_table_begin);
+    ilog_table_begin = NULL;
+  }
 }
 
 int rs_galois_mult(int x, int y)
